/*
 * Copyright (c) 2016 Manuel Braga <mul.braga@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sub license,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <stdlib.h>
#include <stdio.h>
#include <libavformat/avformat.h>
#include "common.h"
#define ERET(...) do { fprintf(stderr, "mux: " __VA_ARGS__); exit(1); } while(0)

static AVFormatContext *fc = NULL;

void mux_init(const char *outfile)
{
	av_register_all();

	fc = avformat_alloc_context();
	if(!fc) ERET("alloc context, failed.\n");

	fc->oformat = av_guess_format("matroska", NULL, NULL);
	if(!fc->oformat) ERET("guess format, failed.\n");

	strncpy(fc->filename, outfile, sizeof(fc->filename));

	if(avio_open(&fc->pb, fc->filename, AVIO_FLAG_WRITE) < 0)
		ERET("io open, failed.\n");


	AVStream *s = NULL;
	s = avformat_new_stream(fc, NULL);
	if(!s) ERET("new stream, failed.\n");

	AVCodecContext *c = s->codec;
	c->codec_type = AVMEDIA_TYPE_VIDEO;
	c->codec_id = CODEC_ID_H264;
	c->width = 16;
	c->height = 16;
	c->bit_rate = 1000000;
	c->time_base.num = 1000 / 25;
	c->time_base.den = 1000;
	c->pix_fmt = PIX_FMT_YUV420P;
	if(fc->oformat->flags & AVFMT_GLOBALHEADER)
		c->flags |= CODEC_FLAG_GLOBAL_HEADER;
	c->flags |= CODEC_FLAG_BITEXACT;


}

void mux_write_header(uint8_t *extradata, int extrasize)
{
	if(!fc) ERET("fc is NULL.\n");

	AVCodecContext *c = fc->streams[0]->codec;

	if(extradata) {
		c->extradata = av_malloc(extrasize);
		memcpy(c->extradata, extradata, extrasize);
		c->extradata_size = extrasize;
	}

	/*av_dump_format(fc, 0, fc->filename, 1);*/
	avformat_write_header(fc, NULL);
}

void mux_write(uint8_t *data, int size, int keyframe)
{
	if(!fc) ERET("fc is NULL.\n");

	AVPacket p;
	av_init_packet(&p);
	p.data = data;
	p.size = size;
	if(keyframe) p.flags |= AV_PKT_FLAG_KEY;

	if(av_interleaved_write_frame(fc, &p) < 0)
		ERET("write frame, failed.\n");

	av_free_packet(&p);
}

void mux_close(void)
{
	if(!fc) ERET("fc is NULL.\n");

	av_write_trailer(fc);

	avio_close(fc->pb);
	avformat_free_context(fc);
}

